/**
 * @file
 *
 * @brief
 *
 * @copyright BSD License (see doc/LICENSE.md or http://www.libelektra.org)
 */

#include <kdb.h>
#include <kdbconfig.h>
#include <kdbmacros.h>
#include <kdbplugin.h>

#include <string.h>

#include <kdberrors.h>

#define STRINGIFY(x) STRINGIFY2 (x)
#define STRINGIFY2(x) #x

int elektraConstantsGet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned ELEKTRA_UNUSED, Key * parentKey ELEKTRA_UNUSED);
int elektraConstantsSet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned ELEKTRA_UNUSED, Key * parentKey ELEKTRA_UNUSED);

static void elektraAddKeyValue (KeySet * ks, Key * parentKey, const char * key, const char * value)
{
	Key * k = keyDup (parentKey);
	keyAddName (k, key);
	keySetString (k, value);
	ksAppendKey (ks, k);
}

static void elektraAddConstants (KeySet * ks, Key * parentKey)
{
	Key * key = keyDup (parentKey);
	ksAppendKey (ks, key);

	key = keyDup (parentKey);
	keyAddName (key, "version");
	keySetString (key, "Duplicate of version info also found below system/elektra/version");
	ksAppendKey (ks, key);


	key = keyDup (key);
	keyAddName (key, "version/KDB_VERSION");
	keySetString (key, "0.8.19");
	ksAppendKey (ks, key);

	key = keyDup (parentKey);
	keyAddName (key, "version/KDB_VERSION_MAJOR");
	keySetString (key, "0");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "version/KDB_VERSION_MINOR");
	keySetString (key, "8");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "version/KDB_VERSION_MICRO");
	keySetString (key, "19");
	ksAppendKey (ks, key);


	elektraAddKeyValue (ks, parentKey, "version/SO_VERSION", "4");
	elektraAddKeyValue (ks, parentKey, "version/SO_VERSION_TOOLS", "2");
	elektraAddKeyValue (ks, parentKey, "version/SO_VERSION_GETENV", "0");


	key = keyDup (parentKey);
	keyAddName (key, "cmake");
	keySetString (key, "All cmake variables as defined in the file cmake/ElektraCache.cmake");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/PLUGINS");
	keySetString (key, "augeas;base64;blockresolver;boolean;c;cachefilter;ccode;conditionals;constants;counter;crypto_botan;crypto_gcrypt;crypto_openssl;csvstorage;curlget;dbus;desktop;doc;dpkg;dump;enum;error;fcrypt;filecheck;fstab;gitresolver;glob;hexcode;hidden;hosts;iconv;ini;iterate;jni;journald;keytometa;line;lineendings;list;logchange;lua;mathcheck;mozprefs;network;ni;noresolver;null;passwd;path;profile;python;python2;regexstore;rename;required;resolver_fm_b_b;resolver_fm_hb_b;resolver_fm_hp_b;resolver_fm_hpu_b;resolver_fm_pb_b;resolver_fm_ub_x;resolver_fm_uhb_xb;resolver_fm_xb_x;resolver_fm_xhp_x;resolver_fm_xp_x;semlock;shell;simpleini;simplespeclang;spec;struct;sync;syslog;tcl;template;timeofday;tracer;type;uname;validation;wresolver;xmltool;yajl");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/TOOLS");
	keySetString (key, "kdb;race;qt-gui");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/KDB_DB_SYSTEM");
	keySetString (key, "/etc/kdb");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/KDB_DB_HOME");
	keySetString (key, "/home");
	ksAppendKey (ks, key);

	elektraAddKeyValue (ks, parentKey, "cmake/KDB_DB_SPEC", "share/elektra/specification");
	elektraAddKeyValue (ks, parentKey, "cmake/KDB_DB_DIR", ".dir");
	elektraAddKeyValue (ks, parentKey, "cmake/KDB_DB_FILE", "default.ecf");
	elektraAddKeyValue (ks, parentKey, "cmake/KDB_DB_INIT", "elektra.ecf");
	elektraAddKeyValue (ks, parentKey, "cmake/KDB_DEFAULT_RESOLVER", "resolver_fm_hpu_b");
	elektraAddKeyValue (ks, parentKey, "cmake/KDB_DEFAULT_STORAGE", "dump");

	key = keyDup (parentKey);
	keyAddName (key, "cmake/KDB_DB_USER");
	keySetString (key, ".config");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/GTEST_ROOT");
	keySetString (key, "");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/BUILD_SHARED");
	keySetString (key, "ON");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/BUILD_FULL");
	keySetString (key, "OFF");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/BUILD_STATIC");
	keySetString (key, "ON");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/BINDINGS");
	keySetString (key, "cpp;glib;gi_lua;gi_python;jna;swig_lua;swig_python2;swig_python");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/TARGET_INCLUDE_FOLDER");
	keySetString (key, "elektra");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/TARGET_CMAKE_FOLDER");
	keySetString (key, "/usr/share/cmake/Modules");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/TARGET_PLUGIN_FOLDER");
	keySetString (key, "elektra");
	ksAppendKey (ks, key);

	elektraAddKeyValue (ks, parentKey, "cmake/ENABLE_DEBUG", "OFF");
	elektraAddKeyValue (ks, parentKey, "cmake/ENABLE_LOGGER", "OFF");

	elektraAddKeyValue (ks, parentKey, "cmake/BUILTIN_PLUGIN_FOLDER", BUILTIN_PLUGIN_FOLDER);
	elektraAddKeyValue (ks, parentKey, "cmake/BUILTIN_DATA_FOLDER", BUILTIN_DATA_FOLDER);
	elektraAddKeyValue (ks, parentKey, "cmake/BUILTIN_EXEC_FOLDER", BUILTIN_EXEC_FOLDER);

	key = keyDup (parentKey);
	keyAddName (key, "cmake/TARGET_PKGCONFIG_FOLDER");
	keySetString (key, "pkgconfig");
	ksAppendKey (ks, key);

	elektraAddKeyValue (ks, parentKey, "cmake/TARGET_DOCUMENTATION_TEXT_FOLDER", "share/doc/elektra");
	elektraAddKeyValue (ks, parentKey, "cmake/TARGET_DOCUMENTATION_HTML_FOLDER", "share/doc/elektra-api/html");
	elektraAddKeyValue (ks, parentKey, "cmake/TARGET_DOCUMENTATION_MAN_FOLDER", "share/man/man3");
	elektraAddKeyValue (ks, parentKey, "cmake/TARGET_DOCUMENTATION_LATEX_FOLDER", "share/doc/elektra-api/latex");

	key = keyDup (parentKey);
	keyAddName (key, "cmake/TARGET_TOOL_EXEC_FOLDER");
	keySetString (key, "lib64/elektra/tool_exec");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/TARGET_TEST_DATA_FOLDER");
	keySetString (key, "share/elektra/test_data");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/TARGET_TEMPLATE_FOLDER");
	keySetString (key, "share/elektra/templates");
	ksAppendKey (ks, key);

	key = keyDup (parentKey);
	keyAddName (key, "cmake/LIB_SUFFIX");
	keySetString (key, "64");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "cmake/CMAKE_INSTALL_PREFIX");
	keySetString (key, "/usr");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "compiler");
	keySetString (key, "Flags defined for compilers as defined in ElektraCompiling.cmake");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "compiler/c_flags");
	keySetString (key, " -std=gnu99  -Wno-deprecated-declarations  -Wno-long-long -pedantic -Wno-variadic-macros -Wall -Wextra -Wno-overlength-strings -Wsign-compare -Wfloat-equal -Wformat-security -Wshadow -Wcomments -Wtrigraphs -Wundef -Wuninitialized -Winit-self -Wmaybe-uninitialized -Wsign-compare -Wfloat-equal -Wformat-security");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "compiler/id");
	keySetString (key, "GNU");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "compiler/coverage");
	keySetString (key, "OFF");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "compiler/cxx_flags");
	keySetString (key, " -std=c++11  -Wno-deprecated-declarations  -Wold-style-cast -Wstrict-null-sentinel -D_GLIBCXX_USE_NANOSLEEP -Wno-missing-field-initializers -Woverloaded-virtual  -Wsign-promo  -Wno-long-long -pedantic -Wno-variadic-macros -Wall -Wextra -Wno-overlength-strings -Wsign-compare -Wfloat-equal -Wformat-security -Wshadow -Wcomments -Wtrigraphs -Wundef -Wuninitialized -Winit-self -Wmaybe-uninitialized");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "compiler/pic_flags");
	keySetString (key, "-fPIC");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "compiler/static_flags");
	keySetString (key, "");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "macros");
	keySetString (key, "Define the macros from kdbos.h and kdbconfig.h");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "macros/KDB_MAX_PATH_LENGTH");
	keySetString (key, STRINGIFY (KDB_MAX_PATH_LENGTH));
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "macros/KDB_FILE_MODE");
	keySetString (key, STRINGIFY (KDB_FILE_MODE));
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "macros/KDB_DIR_MODE");
	keySetString (key, STRINGIFY (KDB_DIR_MODE));
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "macros/KDB_PATH_SEPARATOR");
	keySetString (key, STRINGIFY (KDB_PATH_SEPARATOR));
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "macros/KDB_PATH_ESCAPE");
	keySetString (key, STRINGIFY (KDB_PATH_ESCAPE));
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "package");
	keySetString (key, "Infos about the package as it is installed");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "package/prefix");
	keySetString (key, "/usr");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "package/libdir");
	keySetString (key, "/usr/lib64");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "package/includedir");
	keySetString (key, "/usr/include/elektra");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "package/plugindir");
	keySetString (key, "/usr/lib64/elektra");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "package/tool_execdir");
	keySetString (key, "/usr/lib64/elektra/tool_exec");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "package/templatedir");
	keySetString (key, "/usr/share/elektra/templates");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "package/libs");
	keySetString (key, "-L/usr/lib64 -lelektra");
	ksAppendKey (ks, key);


	key = keyDup (parentKey);
	keyAddName (key, "package/cflags");
	keySetString (key, "-I/usr/include/elektra");
	ksAppendKey (ks, key);
}

int elektraConstantsGet (Plugin * handle ELEKTRA_UNUSED, KeySet * ks, Key * parentKey)
{
	if (!strcmp (keyName (parentKey), "system/elektra/modules/constants"))
	{
		ksAppendKey (ks, keyNew ("system/elektra/modules/constants", KEY_VALUE, "constants plugin waits for your orders", KEY_END));
		ksAppendKey (ks, keyNew ("system/elektra/modules/constants/exports", KEY_END));
		ksAppendKey (ks, keyNew ("system/elektra/modules/constants/exports/get", KEY_FUNC, elektraConstantsGet, KEY_END));
		ksAppendKey (ks, keyNew ("system/elektra/modules/constants/exports/set", KEY_FUNC, elektraConstantsSet, KEY_END));
		KeySet * contract =
			ksNew (30,
#include "readme_constants.c"
			       keyNew ("system/elektra/modules/constants/infos/version", KEY_VALUE, PLUGINVERSION, KEY_END), KS_END);
		ksAppend (ks, contract);
		ksDel (contract);
		return 1;
	}

	elektraAddConstants (ks, parentKey);

	return 1; /* success */
}

int elektraConstantsSet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned, Key * parentKey)
{
	KeySet * info = ksNew (0, KS_END);
	elektraAddConstants (info, parentKey);
	ELEKTRA_SET_ERROR_READ_ONLY (info, returned, parentKey);
	return 0;
}

Plugin * ELEKTRA_PLUGIN_EXPORT (constants)
{
	// clang-format off
	return elektraPluginExport("constants",
		ELEKTRA_PLUGIN_GET,	&elektraConstantsGet,
		ELEKTRA_PLUGIN_SET,	&elektraConstantsSet,
		ELEKTRA_PLUGIN_END);
}

